/*                                                
 *   This is a tutorial for the DECtalk PC.
 *
 *   This program is included to provide an example
 * of coding to a DECtalk-PC under Linux.
 * It is not intended to be functionally
 * or architecturally complete. 
 *
 * This code is based on the dtdemo.c file from DEC found 
 * in the pc1v42b.exe DOS driver package.
 * Modified by Nicolas Pitre, Oct 23, 1997
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>
#include <termios.h>

#include "dtpc_ioctl.h"


/* define global variables */ 

char dectalk_device[] = "/dev/dtpc0";

char paul[] = "    Hi, my name is Paul.  I am the default voice for DECtalk. \n";
char harry[] = "    Hello, my name is Harry.  I have a deep voice. \n";
char frank[] = "    Hi, I'm Frank. I am a senior citizen. \n";
char dennis[] = "    Hello, my name is Dennis.  I have a hoarse voice. \n";
char betty[] = "    Hi, I am Betty.  I'm a middle-aged woman. \n";
char ursula[] = "    I am Ursula.  I'm an older woman. \n";
char rita[] = "    Hello.  I am Rita.  I have a rough voice. \n";
char wendy[] = "    Hi, my name is Wendy.  I have a whispery voice. \n";
char kit[] = "    Hey!  My name is Kit.  I'm eight years old. \n";

char song[] = "[:phone arpa on][:name paul  peh<400,25> niy<400,23>z ih<400,20>n ah<400,18> striy<1200,20>m, <400>fao<400,25> lih<400,23>nx] ";
char song2[] = "[liy<500,20>vz, <100> ah<200,16>v sih<200,18> kah<200,20> mao<1000,12>rx, <200>]";
char song3[] = "[muw<400,20>n lay<400,18>t ih<400,16>n vrr<400,13> maa<1200,16>nt.]";

char clist[] = " \n\
  To run a typing demonstration, type t.  \n\
  To enter text string, type e. \n\
  To repeat text entered, type r. \n\
  For volume up, type u. \n\
  For volume down, type d. \n\
  To set volume to a value between 0 and 100, type b. \n\
  To speak a canned message, type m. \n\
  To flush speech, type f. \n\
  To hear the different voices of DECtalk, type v. \n\
  To hear DECtalk sing, type s. \n\
  To pause and resume speech, press the Space Bar. \n\
  To quit, type q. \n";

char voices[] = " \n\
  To hear Paul's voice, type p. \n\
  To hear Harry 's voice, type h. \n\
  To hear Frank 's voice, type f. \n\
  To hear Dennis 's voice, type d. \n\
  To hear Betty 's voice, type b. \n\
  To hear Ursula 's voice, type u. \n\
  To hear Rita 's voice, type r. \n\
  To hear Wendy 's voice, type w. \n\
  To hear Kit 's voice, type k. \n\
  To hear all the voices, type a. \n\
  To go back to the main menu, type m. \n";

char next[] = " \n\
  Type in the first letter of the voice you want to hear. \n\
  If you would like the list of voices, type v. \n\
  To go to the main menu, type m. \n";

char editbuf[256];		/* used in edit and repeat command */
char rate[] = "[:ra 400]";	/* load change of speaking rate */
char canned1[] = "[:nh][:ra 200] ";
char canned2[] = "Hello, I'm DECtalk, the state of the art in text to speech synthesis. \n";

char setup[] = " \n\
  If you would like the commands spoken, press enter, \n\
  If you would like the commands not to be spoken, press the space-bar. \n";

char list[] = "For a list of commands, type question-mark. ";
char typing[] = "Typing demonstration, type ENTER to exit. ";
char incr[] = "   Incremented the volume by 5. ";
char decr[] = "   Decremented the volume by 5. ";
char volset[] = "Volume setting incorrect, ignoring request. ";
char setting[] = "Setting the volume to requested input. ";
char edit[] = "Press ENTER to end input, type up to 500 characters of text. ";
char valid[] = "Not a valid option. ";
char sayc[] = "[:say clause]";	/* say clause */
char sayl[] = "[:sa le][:ra 400]";	/* say letter at different rate than speech */
char invol[] = "Input volume setting:";
char backs[] = "backspace\xb";		/* used so DECtalk says 'space', 'delete', 'backspace' */
char space[] = "space\xb";
char dels[] = "delete\xb";
char dexit[] = "Exiting DECtalk demo. ";
char defalt[] = "Incorrect command, please type a question-mark for a list of commands. ";
char reset[] = "[:np] [:sa c][:pu so][:ra 210]";
char freset[] = "[:np][:error ignore] [:say cl][:phon arpa on][:ra 210]\xb";
char voicesetup[] = "[:np ]";	


int dtk_fd;		/* DECtalk device file handler */


void send( char *s )
{
  write( dtk_fd, s, strlen(s) ); 	/* send string */
  write( dtk_fd, "\xb", 1 ); 		/* force speech */
}


void flush( void )
{
#if 0
  write( dtk_fd, "\003", 1);
#else
  ioctl( dtk_fd, DTFLUSH, 0 );
#endif
}


void flush_char( unsigned char c )
{
#if 0
  write( dtk_fd, "\003[:say letter]", 14 );
  write( dtk_fd, &c, 1 );
  write( dtk_fd, "[:say cl]\xb", 10 );
#else
  ioctl( dtk_fd, DTFLUSHCHAR, c );
#endif
}


int set_vol (int vol)
{
#if 0
  char buf[16];
  sprintf( buf, "[:vol set %d]\xb", vol );
  send( buf );
#else
  ioctl( dtk_fd, DTVOLSET, vol );
#endif
  return 1;
}


int up_vol (void)
{
#if 0
  send( "[:vol up]\xb" );
#else
  ioctl( dtk_fd, DTVOLUP, 5 );
#endif
  return 1;
}


int dwn_vol (void)
{
#if 0
  send( "[:vol down]\xb" );
#else
  ioctl( dtk_fd, DTVOLDOWN, 5 );
#endif
  return 1;
}


void main (void) 
{
    char chbuf[30];
    char inp, inchar;		/* in character */
    int go = 1;			/* user hasn't typed 'q' for quit yet */
    int i = 0;			/* counter */
    int tc = 0;			/* index counter for edit buf array */
    int vol;			/* inputed volume */
    int spoken = 0;		/* user doesn't want commands spoken */
    int hear;
    struct termios old, new;
    
    /* open the DECtalk device */
    dtk_fd = open( dectalk_device, O_RDWR );
    if( dtk_fd < 0 ){
      perror( dectalk_device );
      exit( -1 );
    }

    /* set tty flags to get characters one by one */
    tcgetattr(0, &old);
    new = old;
    new.c_lflag &= ~(ICANON | ECHO | ISIG);
    new.c_iflag = 0;
    new.c_cc[VMIN] = 1;
    new.c_cc[VTIME] = 1;    /* 0.1 sec intercharacter timeout */
    tcsetattr(0, TCSAFLUSH, &new);

    flush();
    send (reset);
    printf( setup );
    do {
        flush ();			/* asynchronously flushes buffer */
	send( setup );  /* Is a spoken setup desired */
	inp = getchar ();
	if (inp == '\r') spoken = 1;	/* user does want commands spoken */
    } while (!(inp == ' ' || inp == '\r'));

    inp = 0;
    flush ();
  
    printf ("%s\n", list);	/* prompt if want to see list of commands */
    if (spoken) send( list );
  
    while (go) {	/* have not hit 'q' yet */
	send( reset );
	printf( "> " );
	do {
	    inp = tolower (getchar ());	/* Get a command character from the keyboard */
	    if( inp == ' ' ) {
		/* pause speech */
		ioctl( dtk_fd, DTPAUSE, 0 );
		inp = tolower (getchar ());	/* Get a command character from the keyboard */
		ioctl( dtk_fd, DTRESUME, 0 );  /* resume speech */
	    }
	} while( inp == ' ' );

	printf( "%c\n", inp );
	if (spoken) flush_char (inp);
	else flush ();

	if (inp != 0) {	/* if valid command exists process it */
	    switch (inp) {	/* do command */
	    case 't':		/* Typing demo.  uses the flush_char function */
		printf ("%s\n", typing);
		if (spoken) send (typing);
		send (rate);   /* for fast echo */
		do {
		    inchar = getchar ();	/* get character from keyboard until an '*' */
		    flush_char (inchar); /* used in typing mode to speak faster */
	        } while (inchar != '\r');
		putchar ('\n');
		inp = 0;
		break;
	      
	    case 'b':		/* Set the volume to a specific level */
		printf ( "%s\n", invol);	/* Prompt */
		if (spoken) send (invol);
		i = 0;
		while ((chbuf[i] = getchar ()) != '\r') {
		    putchar (chbuf[i]);		/* Echo input */
		    if (spoken) flush_char (chbuf[i]);	/* Audible echo */
		    i++;
		}
		chbuf[i] = 0;
		putchar( '\n' );
		vol = atoi (chbuf);	/* Convert to a number */
		if ((vol > 100) || vol < 0) {
		    printf ("%s,%d\n", volset, vol);
		    if (spoken) send (volset);
		    break;
		}
		if (set_vol (vol) ) {
		    printf ("Setting the volume to requested input.\n");
		    if (spoken) send (setting);
		    inp = 0;
		}
		break;
	      
	    case 'e':	/* Allow user to type in text, send to DECtalk */
		printf ("%s\n", edit);
		if (spoken) send (edit);
		tc = 0;
		send (rate);   /* changes speaking rate */
		do {
		    inchar = getchar ();	/* get character from keyboard until an '*' */
		    editbuf[tc++] = inchar;
		    putchar( inchar );
		    flush_char (inchar); 
		    if (tc && inchar == '\b') tc--;
		} while (inchar != '\r');
		editbuf[tc++] = ' ';
		editbuf[tc] = 0;
		send (reset);
		send (editbuf); 	/* send string to DECtalk */
		putchar( '\n' );
		inp = 0;
		break;
	      
	    case 'r':		/* repeat text entered */
		send (editbuf);
		inp = 0;
		break;
	      
	    case 'u':		/* Increment the volume by 1. */
		if (up_vol () != -1 ) {
		    if (spoken) send (incr);
		    printf ("%s\n", incr);
		    inp = 0;
		}
		break;
	      
	    case 'd':		/* Decrease the volume by 1. */
		if (dwn_vol () != -1) {
		    if (spoken) send (decr);
		    printf ("%s\n", decr);
		    inp = 0;
		}
		break;
	      
	    case 'v':		/* open the correct file to hear the voices */
		printf ("%s\n", voices);	/* Prompt */
		if (spoken) send (voices); 
		i = 0;
		hear = 1;
		do {
		    printf ("  >>");
		    inp = getchar ();
		    printf ("%c\n", inp);
		    if (spoken) flush_char ((char) inp);
		    switch (inp) {	/* Speak non-printing char's of interest */
		    case 'v':
			  printf ("%s\n", voices);	/* Prompt */
			  if (spoken) send (voices);
			  break;
		      
		    case 'p':
			  printf ("%s", paul);
			  voicesetup[3] = 'p';
			  send (voicesetup);
			  send (paul);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;

		    case 'h':
			  printf ("%s", harry);
			  voicesetup[3] = 'h';
			  send (voicesetup);
			  send (harry);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'f':
			  printf ("%s", frank);
			  voicesetup[3] = 'f';
			  send (voicesetup);
			  send (frank);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'd':
			  printf ("%s", dennis);
			  voicesetup[3] = 'd';
			  send (voicesetup);
			  send (dennis);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'b':
			  printf ("%s", betty);
			  voicesetup[3] = 'b';
			  send (voicesetup);
			  send (betty);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'u':
		      	  printf ("%s", ursula);
			  voicesetup[3] = 'u';
			  send (voicesetup);
			  send (ursula);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'r':
			  printf ("%s", rita);
			  voicesetup[3] = 'r';
			  send (voicesetup);
			  send (rita);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'w':
			  printf ("%s", wendy);
			  voicesetup[3] = 'w';
			  send (voicesetup);
			  send (wendy);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'k':
			  printf ("%s", kit);
			  voicesetup[3] = 'k';
			  send (voicesetup);
			  send (kit);
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;
		      
		    case 'a':
			  voicesetup[3] = 'p';
			  send (voicesetup);
			  send (paul);
			  printf ("%s", paul);
			
			  voicesetup[3] = 'h';
			  send (voicesetup);
			  send (harry);
			  printf ("%s", harry);

			  voicesetup[3] = 'f';
			  send (voicesetup);
			  send (frank);
			  printf ("%s", frank);
			
			  voicesetup[3] = 'd';
			  send (voicesetup);
			  send (dennis);
			  printf ("%s", dennis);
			
			  voicesetup[3] = 'b';
			  send (voicesetup);
			  send (betty);
			  printf ("%s", betty);
			
			  voicesetup[3] = 'r';
			  send (voicesetup);
			  send (rita);
			  printf ("%s", rita);
			
			  voicesetup[3] = 'u';
			  send (voicesetup);
			  send (ursula);
			  printf ("%s", ursula);
			
			  voicesetup[3] = 'w';
			  send (voicesetup);
			  send (wendy);
			  printf ("%s", wendy);
			
			  voicesetup[3] = 'k';
			  send (voicesetup);
			  send (kit);
			  printf ("%s", kit);
			
			  printf ("%s", next);
			  if (spoken) send (next);
			  break;

		    case 'm':
			  hear = 0;
			  inp = 0;
			  break;

		    default:
			  printf ("%s\n", valid);
			  if (spoken) send (valid);
			  break;
		    }
		} while (hear);
		break;
	      
	    case 's':		/* open song file and send to DECtalk */
		send (song);
		send (song2);
		send (song3);
		inp = 0;
		break;
	      
	    case 'f':		/* Stop speaking and FLUSH */
		flush ();
		inp = 0;
		break;
	      
	    case 'm':		/* Send a buffer of data (canned message) */
		send (canned1);
		send (canned2);
		printf ("%s", canned2);		/* buffer sent so print it */
		inp = 0;
		break;
	      
	    case 'q':		/* QUIT this program */
		go = 0;
		if (spoken) send (dexit);
		inp = 0;
		break;
	      
	    case '?':		/* print list of commands */
		if (spoken) send (clist);
		printf ("%s", clist);
		inp = 0;
		break;
	      
	    default:
		if (spoken) send (defalt);
		printf ("%s", defalt);
		inp = 0;
		break;
	    }
	}
    }
  
    tcsetattr(0, TCSAFLUSH, &old);
    close (dtk_fd);
}




