/*
 * This is the DECtalk PC driver for the Linux kernel, version 0.91
 *
 * Original 386BSD source:
 *      Copyright (c) 1996 Brian Buhrow <buhrow@lothlorien.nfbcal.org>
 *
 * Rewritten for Linux:
 *      Copyright (c) 1997 Nicolas Pitre <nico@cam.org>
 *
 * Maintained by:
 *      Nicolas Pitre <nico@cam.org>
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _DTPC_IOCTL_H
#define _DTPC_IOCTL_H

#ifndef __KERNEL__
#include <sys/ioctl.h> /* get standard ioctl definitions */
#endif

/*
 * This structure is the one we use for passing the kernel into the card
 */
struct dt_kerninfo {
    char  *dt_start; /* start of user space buffer */
    unsigned int dt_len; /* how long is the buffer */
    unsigned short dt_csval; /* used for  ansilary address information */
    unsigned short dt_ipval; /* more address info for kernel loading */
};

/* the loadinfo structure tells the kernel how to load data into the card 
 * from user space.  
 */
struct dt_loadinfo {
    char *dtl_start; /* the user space data buffer */
    unsigned int dtl_len; /* how large is the buffer */
    unsigned long dtl_addr; /* address on the card to load into */
};

/*
 * Structure used to set up a dictionary on the card.
 */
struct dt_dicinfo {
    unsigned short dtd_type; /* dictionary type */
    unsigned int dtd_entries; /* how many entries in the dictionary */
    unsigned long dtd_addr; /* address on card where dictionary lives */
};

/* 
 * Structure for starting a newly loaded executable
 */
struct dt_taskinfo {
    unsigned short dts_csval; /* code segment value of start address */
    unsigned short dts_ipval; /* offset into that segment */
};

/* 
 * Actually define the ioctls here
 */

/* These ioctls are ment to be used only by dtload */
#define DTIOLDKERNEL	_IOW( 'd', 0, struct dt_kerninfo )  /* load kernel */
#define DTIOALLOCMEM	_IO( 'd', 1 )  /* reserve memory on card */
#define DTIOLDMEM	_IOW( 'd', 2, struct dt_loadinfo )  /* load memory with data */
#define DTIOSETDICT	_IOW( 'd', 3, struct dt_dicinfo )  /* Set dictionary info */
#define DTIOSTARTEXE	_IOW( 'd', 4, struct dt_taskinfo )
#define DTIOTEST	_IO( 'd', 5 )  /* test ioctl, not necessary */
#define DTIOWARM	_IO( 'd', 6 )  /* determine if the card is already loaded */

/* Those ioctls are for user programs */
#define DTFLUSH 	_IO( 'd', 7 )  /* flush speech */
#define DTFLUSHCHAR	_IO( 'd', 8 )  /* flush speech and say character */
#define DTVOLUP 	_IO( 'd', 9 )  /* increments the volume */
#define DTVOLDOWN	_IO( 'd', 10 )  /* decrements the volume */
#define DTVOLSET	_IO( 'd', 11 )  /* set the volume */
#define DTPAUSE		_IO( 'd', 12 )  /* pause speech */
#define DTRESUME	_IO( 'd', 13 )  /* resume speech */

#endif  /* _DTPC_IOCTL_H */
